"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var node_lmdb_1 = require("node-lmdb");
var chaingun_1 = require("@notabug/chaingun");
exports.DEFAULT_CONFIG = {
    path: 'lmdb'
};
var GunLmdbClient = /** @class */ (function () {
    function GunLmdbClient(lmdbConfig) {
        if (lmdbConfig === void 0) { lmdbConfig = exports.DEFAULT_CONFIG; }
        this.lmdb = node_lmdb_1.default;
        this.env = new node_lmdb_1.default.Env();
        this.env.open(lmdbConfig);
        this.dbi = this.env.openDbi({
            name: 'gun-nodes',
            create: true
        });
    }
    GunLmdbClient.prototype.get = function (soul) {
        if (!soul)
            return null;
        var txn = this.env.beginTxn({ readOnly: true });
        try {
            var data = this.deserialize(txn.getStringUnsafe(this.dbi, soul));
            txn.commit();
            return data;
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.getRaw = function (soul) {
        if (!soul)
            return null;
        var txn = this.env.beginTxn({ readOnly: true });
        try {
            var data = txn.getString(this.dbi, soul);
            txn.commit();
            return data || '';
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.serialize = function (node) {
        return JSON.stringify(node);
    };
    GunLmdbClient.prototype.deserialize = function (data) {
        return JSON.parse(data);
    };
    GunLmdbClient.prototype.writeNode = function (soul, nodeData) {
        var _a, _b;
        if (!soul)
            return;
        var txn = this.env.beginTxn();
        try {
            var existingData = txn.getStringUnsafe(this.dbi, soul);
            var node = this.deserialize(existingData) || undefined;
            var existingGraph = (_a = {}, _a[soul] = node, _a);
            var graphUpdates = (_b = {}, _b[soul] = nodeData, _b);
            var graphDiff = chaingun_1.diffGunCRDT(graphUpdates, existingGraph);
            if (!graphDiff || !graphDiff[soul])
                return;
            var updatedGraph = chaingun_1.mergeGraph(existingGraph, graphDiff);
            var updated = updatedGraph[soul];
            txn.putString(this.dbi, soul, this.serialize(updated));
            txn.commit();
            return graphDiff[soul];
        }
        catch (e) {
            txn.abort();
            throw e;
        }
    };
    GunLmdbClient.prototype.write = function (put) {
        if (!put)
            return;
        var diff = {};
        var hasDiff = false;
        for (var soul in put) {
            var nodeDiff = this.writeNode(soul, put[soul]);
            if (nodeDiff) {
                diff[soul] = nodeDiff;
                hasDiff = true;
            }
        }
        if (hasDiff)
            return diff;
    };
    GunLmdbClient.prototype.close = function () {
        this.dbi.close();
        this.env.close();
    };
    return GunLmdbClient;
}());
exports.GunLmdbClient = GunLmdbClient;
//# sourceMappingURL=GunLmdbClient.js.map